from __future__ import annotations
from datetime import datetime, timedelta
from airflow import DAG
from airflow.decorators import task
from apache_airflow_provider_magento.operators.graphql import MagentoGraphQLOperator
from airflow.exceptions import AirflowException
import logging

default_args = {
    'owner': 'airflow',
    'depends_on_past': False,
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

with DAG(
    'magento_graphql_customer',
    default_args=default_args,
    description='A DAG to handle Magento GraphQL operations using TaskFlow API',
    schedule_interval=None,
    start_date=datetime(2024, 8, 22),
    catchup=False,
) as dag:

    @task
    def create_customer() -> dict:
        try:
            operator = MagentoGraphQLOperator(
                task_id='create_customer',
                query="""
                mutation {
                  createCustomer(
                    input: {
                      firstname: "John"
                      lastname: "Doe"
                      email: "john.doe@example.com"
                      password: "Airflow@123"
                    }
                  ) {
                    customer {
                      id
                      firstname
                      lastname
                      email
                    }
                  }
                }
                """,
                magento_conn_id='magento_default',
            )
            result = operator.execute(context={})
            customer_email = result['data']['createCustomer']['customer']['email']
            logging.info(f"Customer email created: {customer_email}")
            return {'customer_email': customer_email}  # Return as a dictionary
        except AirflowException as e:
            logging.error(f"Error in create_customer task: {e}")
            raise

    @task
    def generate_customer_token(customer_email: str) -> str:
        logging.info(f"Customer email received: {customer_email}")
        try:
            operator = MagentoGraphQLOperator(
                task_id='generate_customer_token',
                query=f"""
                mutation {{
                  generateCustomerToken(
                    email: "{customer_email}"
                    password: "Airflow@123"
                  ) {{
                    token
                  }}
                }}
                """,
                magento_conn_id='magento_default',
            )
            result = operator.execute(context={})
            if 'errors' in result:
                raise AirflowException(f"GraphQL errors: {result['errors']}")
            customer_token = result['data']['generateCustomerToken']['token']
            logging.info(f"Customer token generated: {customer_token}")
            return customer_token
        except AirflowException as e:
            logging.error(f"Error in generate_customer_token task: {e}")
            raise

    @task
    def get_customer_info(customer_token: str) -> dict:
        logging.info(f"Customer token received: {customer_token}")
        try:
            operator = MagentoGraphQLOperator(
                task_id='get_customer_info',
                query="""
                query {
                  customer {
                    firstname
                    lastname
                    suffix
                    email
                    addresses {
                      firstname
                      lastname
                      street
                      city
                      region {
                        region_code
                        region
                      }
                      postcode
                      country_code
                      telephone
                    }
                  }
                }
                """,
                magento_conn_id='magento_default',
                headers={
                    'Authorization': f'Bearer {customer_token}'
                }
            )
            result = operator.execute(context={})
            if 'errors' in result:
                raise AirflowException(f"GraphQL errors: {result['errors']}")
            return result
        except AirflowException as e:
            logging.error(f"Error in get_customer_info task: {e}")
            raise

    # Define task dependencies
    customer_creation_result = create_customer()
    customer_email = customer_creation_result['customer_email']  # Extract email from the dictionary
    customer_token = generate_customer_token(customer_email)
    get_customer_info(customer_token)




from __future__ import annotations
from airflow import DAG
from airflow.decorators import task
from datetime import datetime
from apache_airflow_provider_magento.operators.rest import MagentoRestOperator

default_args = {
    'owner': 'airflow',
    'depends_on_past': False,
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 1,
}

def generate_product_name(sku: str) -> str:
    return f"Product Name - {sku}"

@task
def create_product(sku: str, price: float, **kwargs) -> str:
    product_name = generate_product_name(sku)
    product_data = {
        "product": {
            "sku": sku,
            "name": product_name,
            "price": price,
            "status": 1,
            "type_id": "simple",
            "attribute_set_id": 4,
            "weight": 1,
            "extension_attributes": {
                "stock_item": {
                    "qty": 450,
                    "is_in_stock": True
                }
            }
        }
    }
    op = MagentoRestOperator(
        task_id=f'create_product_{sku}_op',
        endpoint='products',
        method='POST',
        data=product_data,
    )
    response = op.execute(context=kwargs)
    product_id = response['sku']
    return product_id

@task
def check_product_exists(sku: str, **kwargs) -> bool:
    op = MagentoRestOperator(
        task_id=f'check_product_exists_{sku}_op',
        endpoint=f'products/{sku}',
        method='GET',
        data={},
    )
    try:
        response = op.execute(context=kwargs)
        return True  # Product exists
    except Exception as e:
        if '404' in str(e):
            return False
        else:
            raise

@task
def create_guest_cart(**kwargs) -> str:
    op = MagentoRestOperator(
        task_id='create_guest_cart_op',
        endpoint='guest-carts',
        method='POST',
        data={},
    )
    response = op.execute(context=kwargs)
    quote_id = response
    return quote_id

@task
def add_item_to_cart(product_id: str, quote_id: str, **kwargs) -> dict:
    cart_item = {
        "cartItem": {
            "sku": product_id,
            "qty": 1,
            "name": generate_product_name(product_id),
            "price": 100.00,
            "product_type": "simple",
            "quote_id": quote_id
        }
    }
    op = MagentoRestOperator(
        task_id=f'add_item_{product_id}_to_cart_op',
        endpoint=f'guest-carts/{quote_id}/items',
        method='POST',
        data=cart_item,
    )
    response = op.execute(context=kwargs)
    return {
        "quote_id": quote_id,
        "item_id": response.get('item_id')
    }

@task
def set_billing_address(quote_id: str, **kwargs) -> str:
    op = MagentoRestOperator(
        task_id='set_billing_address_op',
        endpoint=f'guest-carts/{quote_id}/billing-address',
        method='POST',
        data={
            "address": {
                "region_code": "CA",
                "region": "California",
                "region_id": 12,
                "country_id": "US",
                "street": ["123 Main Street"],
                "postcode": "90210",
                "city": "Beverly Hills",
                "telephone": "1234567890",
                "firstname": "John",
                "lastname": "Doe",
                "middlename": "",
                "prefix": "",
                "suffix": "",
                "vat_id": "",
                "company": "",
                "fax": ""
            }
        },
    )
    response = op.execute(context=kwargs)
    return quote_id

@task
def set_shipping_address(quote_id: str, **kwargs) -> str:
    op = MagentoRestOperator(
        task_id='set_shipping_address_op',
        endpoint=f'guest-carts/{quote_id}/shipping-information',
        method='POST',
        data={
            "addressInformation": {
                "shipping_address": {
                    "region_code": "CA",
                    "region": "California",
                    "region_id": 12,
                    "country_id": "US",
                    "street": ["123 Main Street"],
                    "postcode": "90210",
                    "city": "Beverly Hills",
                    "telephone": "1234567890",
                    "firstname": "John",
                    "lastname": "Doe",
                    "middlename": "",
                    "prefix": "",
                    "suffix": "",
                    "vat_id": "",
                    "company": "",
                    "fax": ""
                },
                "shippingCarrierCode": "flatrate",
                "shippingMethodCode": "flatrate"
            }
        },
    )
    response = op.execute(context=kwargs)
    return quote_id

@task
def set_payment_method_and_place_order(quote_id: str, **kwargs) -> str:
    op = MagentoRestOperator(
        task_id='place_order_op',
        endpoint=f'guest-carts/{quote_id}/payment-information',
        method='POST',
        data={
            "email": "guest@example.com",
            "paymentMethod": {"method": "checkmo"}
        },
    )
    order_id = op.execute(context=kwargs)
    return order_id

@task
def get_order_info(order_id: str, **kwargs) -> dict:
    op = MagentoRestOperator(
        task_id='get_order_info_op',
        endpoint=f'orders/{order_id}',
        method='GET',
        data={},
    )
    response = op.execute(context=kwargs)
    return response

@task
def create_invoice(order_id: str, order_items: list, **kwargs) -> str:
    items = [{"order_item_id": item["item_id"], "qty": item["qty_ordered"]} for item in order_items]
    op = MagentoRestOperator(
        task_id='create_invoice_op',
        endpoint=f'order/{order_id}/invoice',
        method='POST',
        data={
            "capture": True,
            "items": items,
            "notify": True,
            "appendComment": True,
            "comment": {
                "comment": "Invoice created",
                "is_visible_on_front": 0
            },
            "arguments": {}
        },
    )
    response = op.execute(context=kwargs)
    invoice_id = response
    return invoice_id

@task
def create_shipment(order_id: str, order_items: list, **kwargs) -> str:
    items = [{"order_item_id": item["item_id"], "qty": item["qty_ordered"]} for item in order_items]
    op = MagentoRestOperator(
        task_id='create_shipment_op',
        endpoint=f'order/{order_id}/ship',
        method='POST',
        data={
            "items": items,
            "notify": True,
            "appendComment": True,
            "comment": {
                "comment": "Shipment created",
                "is_visible_on_front": 0
            },
            "tracks": [
                {
                    "track_number": "12345",
                    "title": "Tracking",
                    "carrier_code": "carrier"
                }
            ],
            "packages": [{}]
            
        },
    )
    response = op.execute(context=kwargs)
    shipment_id = response
    return shipment_id

with DAG(
    'magento_guest_order_rest_demo',
    default_args=default_args,
    description='A DAG to perform various operations on Magento',
    schedule_interval=None,  # Run only once manually
    start_date=datetime(2024, 8, 19),
    catchup=False,
) as dag:

    # Define product SKUs and prices
    sku_1 = 'product_sku_1'
    sku_2 = 'product_sku_2'
    price_1 = 100.00
    price_2 = 150.00

    # Define tasks
    product_exists_1 = check_product_exists(sku_1)
    product_exists_2 = check_product_exists(sku_2)
    create_product_1 = create_product(sku_1, price_1)
    create_product_2 = create_product(sku_2, price_2)
    quote_id = create_guest_cart()
    item_1 = add_item_to_cart(product_id=sku_1, quote_id=quote_id)
    item_2 = add_item_to_cart(product_id=sku_2, quote_id=quote_id)
    quote_with_billing = set_billing_address(quote_id)
    quote_with_shipping = set_shipping_address(quote_with_billing)
    order_id = set_payment_method_and_place_order(quote_with_shipping)
    order_info = get_order_info(order_id)
    invoice_id = create_invoice(order_id, order_info['items'])
    shipment_id = create_shipment(order_id, order_info['items'])

    # Set task dependencies
    product_exists_1 >> create_product_1
    product_exists_2 >> create_product_2
    create_product_1 >> quote_id
    create_product_2 >> quote_id
    quote_id >> item_1 >> item_2 >> quote_with_billing >> quote_with_shipping >> order_id >> order_info >> invoice_id >> shipment_id




from __future__ import annotations
from airflow.models.baseoperator import BaseOperator
from apache_airflow_provider_magento.hooks.magento import MagentoHook
from airflow.exceptions import AirflowException

class MagentoGraphQLOperator(BaseOperator):
    """Operator for executing GraphQL API requests to Magento."""

    def __init__(self, query: str, variables: dict = None, magento_conn_id: str = "magento_default", headers: dict = None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.query = query
        self.variables = variables or {}
        self.magento_conn_id = magento_conn_id
        self.headers = headers or {}

    def execute(self, context):
        magento_hook = MagentoHook(self.magento_conn_id)

        try:
            result = magento_hook.graphql_request(query=self.query, variables=self.variables, headers=self.headers)
            if 'errors' in result:
                self.log.error("GraphQL errors received: %s", result['errors'])
                raise AirflowException(f"GraphQL errors: {result['errors']}")
            self.log.info("Response received from Magento GraphQL API: %s", result)
            return result
        except AirflowException as ae:
            self.log.error("AirflowException occurred: %s", ae)
            raise
        except ValueError as ve:
            self.log.error("ValueError occurred: %s", ve)
            raise
        except Exception as e:
            self.log.error("An unexpected error occurred: %s", e, exc_info=True)
            raise




from __future__ import annotations

from airflow.models.baseoperator import BaseOperator
from apache_airflow_provider_magento.hooks.magento import MagentoHook
from airflow.exceptions import AirflowException
import logging

class MagentoRestOperator(BaseOperator):
    def __init__(self, 
                 endpoint: str, 
                 method: str = 'GET', 
                 data: dict = None, 
                 search_criteria: dict = None, 
                 headers: dict = None,
                 magento_conn_id: str = "magento_default", 
                 *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = endpoint
        self.method = method.upper()
        self.data = data or {}
        self.search_criteria = search_criteria or {}
        self.headers = headers or {}  # Initialize headers
        self.magento_conn_id = magento_conn_id

        # Validate HTTP method
        if self.method not in ['GET', 'POST', 'PUT', 'DELETE']:
            raise ValueError(f"Unsupported HTTP method: {self.method}")

    def execute(self, context):
        magento_hook = MagentoHook(self.magento_conn_id)  # Instantiate the MagentoHook
        result = None

        try:
            if self.method == 'GET':
                result = magento_hook.get_request(self.endpoint, search_criteria=self.search_criteria, headers=self.headers)
            elif self.method == 'POST':
                result = magento_hook.post_request(self.endpoint, data=self.data, headers=self.headers)
            elif self.method == 'PUT':
                result = magento_hook.put_request(self.endpoint, data=self.data, headers=self.headers)
            elif self.method == 'DELETE':
                result = magento_hook.delete_request(self.endpoint, data=self.data, headers=self.headers)

            self.log.info("Response received from Magento API: %s", result)
            return result

        except AirflowException as ae:
            self.log.error("AirflowException occurred: %s", ae)
            raise

        except ValueError as ve:
            self.log.error("ValueError occurred: %s", ve)
            raise

        except Exception as e:
            self.log.error("An unexpected error occurred: %s", e, exc_info=True)
            raise




from __future__ import annotations

from datetime import datetime

from airflow.models import DAG
from airflow.operators.python import PythonOperator
from apache_airflow_provider_magento.operators.sales import GetOrdersOperator


def process_orders(**kwargs):
    ti = kwargs["ti"]
    orders = ti.xcom_pull(task_ids="get_orders_task", key="magento_orders")
    if orders:
        # Process orders
        for order in orders:
            print(order)
    else:
        print("No orders to process")


dag = DAG(
    dag_id="magento_get_orders",
    start_date=datetime(2024, 8, 18),
    schedule_interval="@daily",
)

get_orders_task = GetOrdersOperator(task_id="get_orders_task", dag=dag, status="pending")

process_orders_task = PythonOperator(
    task_id="process_orders_task",
    python_callable=process_orders,
    provide_context=True,
    dag=dag,
)

get_orders_task >> process_orders_task



from __future__ import annotations

from requests_oauthlib import OAuth1
import requests
from urllib.parse import urlencode
from airflow.exceptions import AirflowException
from airflow.hooks.base import BaseHook
import json

class MagentoHook(BaseHook):
    """Creates a connection to Magento and allows data interactions via Magento's REST API and GraphQL API."""

    conn_name_attr = "magento_conn_id"
    default_conn_name = "magento_default"
    conn_type = "magento"
    hook_name = "Magento"

    BASE_URL = "/rest/default/V1"  # REST API base URL
    GRAPHQL_ENDPOINT = "/graphql"  # GraphQL API endpoint

    def __init__(self, magento_conn_id=default_conn_name):
        super().__init__()
        self.magento_conn_id = magento_conn_id
        self.connection = self.get_connection(self.magento_conn_id)
        self._validate_connection()
        self._configure_oauth()

    def _validate_connection(self):
        """Validate Magento connection configuration."""
        if not self.connection.host:
            raise AirflowException("Magento connection host is not set properly in Airflow connection")
        if not all([self.connection.extra_dejson.get(field) for field in ["consumer_key", "consumer_secret", "access_token", "access_token_secret"]]):
            raise AirflowException("Magento OAuth credentials are not set properly in Airflow connection")

    def _configure_oauth(self):
        """Configure OAuth authentication for Magento API."""
        self.consumer_key = self.connection.extra_dejson.get("consumer_key")
        self.consumer_secret = self.connection.extra_dejson.get("consumer_secret")
        self.access_token = self.connection.extra_dejson.get("access_token")
        self.access_token_secret = self.connection.extra_dejson.get("access_token_secret")
        self.oauth = OAuth1(
            self.consumer_key,
            self.consumer_secret,
            self.access_token,
            self.access_token_secret,
            signature_method='HMAC-SHA256'
        )

    def _get_full_url(self, endpoint):
        """Construct the full URL for Magento API."""
        base_url = self.connection.host
        base_url = base_url if base_url.startswith('http') else f"https://{base_url}"
        base_url = base_url.rstrip('/')  # Ensure no trailing slash
        endpoint_url = endpoint.lstrip('/')  # Ensure no leading slash
        return f"{base_url}{self.BASE_URL}/{endpoint_url}"

    def _get_graphql_url(self):
        """Construct the full URL for Magento GraphQL API."""
        base_url = self.connection.host
        base_url = base_url if base_url.startswith('http') else f"https://{base_url}"
        return f"{base_url}{self.GRAPHQL_ENDPOINT}"

    def _handle_response(self, response):
        """Handle HTTP response, logging errors and raising exceptions if needed."""
        try:
            response.raise_for_status()
            return response.json()
        except requests.exceptions.HTTPError as http_err:
            self._log_error_response(response)
            raise AirflowException(f"HTTP error occurred: {http_err}")
        except requests.exceptions.Timeout as timeout_err:
            self.log.error("Request timed out: %s", timeout_err)
            raise AirflowException(f"Request timed out: {timeout_err}")
        except requests.exceptions.RequestException as req_err:
            self._log_error_response(response)
            raise AirflowException(f"Request failed: {req_err}")
        except json.JSONDecodeError as json_err:
            self.log.error("Failed to decode JSON response: %s", json_err)
            raise AirflowException(f"Failed to decode JSON response: {json_err}")
        except Exception as e:
            self.log.error("An unexpected error occurred: %s", e, exc_info=True)
            raise AirflowException(f"An unexpected error occurred: {e}")

    def _log_error_response(self, response):
        """Log detailed error information from the response."""
        try:
            error_details = response.json()
        except json.JSONDecodeError:
            error_details = {"error": "Failed to decode error details"}
        self.log.error("Response status code: %d", response.status_code)
        self.log.error("Response headers: %s", response.headers)
        self.log.error("Response body: %s", response.text)
        self.log.error("Error details: %s", error_details)

    def _send_request(self, endpoint, method="GET", data=None, search_criteria=None, headers=None):
        """Perform an API request to Magento."""
        url = self._get_full_url(endpoint)

        if search_criteria:
            query_string = urlencode(search_criteria, doseq=True)
            url = f"{url}?{query_string}"

        # Determine if headers contain an authorization token
        if headers and 'Authorization' in headers:
            response = requests.request(method, url, json=data, headers=headers, verify=False)
        else:
            response = requests.request(method, url, auth=self.oauth, json=data, headers=headers, verify=False)

        return self._handle_response(response)

    def get_request(self, endpoint, search_criteria=None, headers=None):
        """Perform a GET API request to Magento."""
        return self._send_request(endpoint, method="GET", search_criteria=search_criteria, headers=headers)

    def post_request(self, endpoint, data=None, headers=None):
        """Perform a POST API request to Magento."""
        return self._send_request(endpoint, method="POST", data=data, headers=headers)

    def put_request(self, endpoint, data=None, headers=None):
        """Perform a PUT API request to Magento."""
        return self._send_request(endpoint, method="PUT", data=data, headers=headers)

    def delete_request(self, endpoint, data=None, headers=None):
        """Perform a DELETE API request to Magento."""
        return self._send_request(endpoint, method="DELETE", data=data, headers=headers)

    def graphql_request(self, query, variables=None, headers=None):
        """Perform a GraphQL API request to Magento."""
        url = self._get_graphql_url()
        payload = {
            'query': query,
            'variables': variables or {}
        }

        # Determine if headers contain an authorization token
        if headers and 'Authorization' in headers:
            response = requests.post(url, json=payload, headers=headers, verify=False)
        else:
            response = requests.post(url, json=payload, auth=self.oauth, headers=headers, verify=False)

        return self._handle_response(response)




from __future__ import annotations
from airflow.models import BaseOperator
from apache_airflow_provider_magento.hooks.magento import MagentoHook

class CreateCustomerOperator(BaseOperator):
    """Create a customer in Magento."""

    def __init__(
        self,
        magento_conn_id="magento_default",
        customer_data=None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.magento_conn_id = magento_conn_id
        self.customer_data = customer_data or {}

    def execute(self, context):
        magento_hook = MagentoHook(magento_conn_id=self.magento_conn_id)
        endpoint = "customers"

        if not self.customer_data:
            raise ValueError("No customer data provided")

        # Create customer
        response = magento_hook.post_request(endpoint, data=self.customer_data)

        # Log the response for debugging
        self.log.info(f"Response from Magento: {response}")

        # Check if the response contains the 'id' key
        if "id" in response:
            customer_id = response.get("id")
            self.log.info(f"Customer created successfully with ID: {customer_id}")
            context["ti"].xcom_push(key="magento_customer_id", value=customer_id)
        else:
            error_message = response.get('message', 'Unknown error')
            self.log.error(f"Failed to create customer: {error_message}")
            raise Exception(f"Failed to create customer: {error_message}")




from __future__ import annotations

from airflow.models import BaseOperator
from apache_airflow_provider_magento.hooks.magento import MagentoHook


class GetOrdersOperator(BaseOperator):
    """Fetch orders based on order status from Magento."""

    def __init__(self, magento_conn_id="magento_default", status="pending", page_size=100, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.magento_conn_id = magento_conn_id
        self.status = status
        self.page_size = page_size

    def execute(self, context):
        magento_hook = MagentoHook(magento_conn_id=self.magento_conn_id)
        endpoint = "orders"
        orders = []
        current_page = 1
        while True:
            search_criteria = {
                "searchCriteria[pageSize]": self.page_size,
                "searchCriteria[currentPage]": current_page,
            }

            if self.status:
                search_criteria.update(
                    {
                        "searchCriteria[filterGroups][0][filters][0][field]": "status",
                        "searchCriteria[filterGroups][0][filters][0][value]": self.status,
                        "searchCriteria[filterGroups][0][filters][0][conditionType]": "eq",
                    }
                )
            data = magento_hook.get_request(endpoint, search_criteria=search_criteria)
            if not data["items"]:
                break

            orders.extend(data["items"])
            current_page += 1

        if orders:
            context["ti"].xcom_push(key="magento_orders", value=orders)
        else:
            self.log.info("No new orders found.")



from airflow import DAG
from airflow.decorators import task
from datetime import datetime

from apache_airflow_provider_magento.operators.rest import MagentoRestOperator

default_args = {
    'owner': 'airflow',
    'retries': 1,
}

dag = DAG(
    'magento_customer_order_rest_demo',
    default_args=default_args,
    description='Magento Customer Order Creation DAG',
    schedule_interval=None,
    start_date=datetime(2024, 8, 22),
    catchup=False,
)

customer_email = 'customer@example.com'
customer_password = 'Airflow@123'

@task
def generate_customer_token(email: str, password: str, **kwargs) -> str:
    op = MagentoRestOperator(
        task_id='generate_customer_token_op',
        endpoint='integration/customer/token',
        method='POST',
        data={
            'username': email,
            'password': password
        }
    )
    response = op.execute(context=kwargs)
    return response  # Customer OAuth token

@task
def check_customer_exists(email: str, **kwargs) -> bool:
    op = MagentoRestOperator(
        task_id='check_customer_exists_op',
        endpoint=f'customers/search?searchCriteria[filterGroups][0][filters][0][field]=email&searchCriteria[filterGroups][0][filters][0][value]={email}',
        method='GET'
    )
    response = op.execute(context=kwargs)
    return len(response.get('items', [])) > 0

@task
def create_customer(email: str, password: str, **kwargs) -> str:
    op = MagentoRestOperator(
        task_id='create_customer_op',
        endpoint='customers',
        method='POST',
        data={
            "customer": {
                "email": email,
                "firstname": "John",
                "lastname": "Doe",
                "website_id": 1,
                "store_id": 1,
                "group_id": 1
            },
            "password": password
        }
    )
    response = op.execute(context=kwargs)
    return response.get('id')

@task
def create_cart(customer_token: str, **kwargs) -> str:
    op = MagentoRestOperator(
        task_id='create_cart_op',
        endpoint='carts/mine',
        method='POST',
        headers={'Authorization': f'Bearer {customer_token}'}
    )
    response = op.execute(context=kwargs)
    return response  # Returns quoteId

@task
def add_product_to_cart(customer_token: str, quote_id: str, sku: str, qty: int, **kwargs):
    op = MagentoRestOperator(
        task_id=f'add_{sku}_to_cart_op',
        endpoint=f'carts/mine/items',
        method='POST',
        headers={'Authorization': f'Bearer {customer_token}'},
        data={
            "cartItem": {
                "quote_id": quote_id,
                "sku": sku,
                "qty": qty
            }
        }
    )
    response = op.execute(context=kwargs)
    return response

@task
def set_billing_shipping_address(customer_token: str, quote_id: str, **kwargs):
    op = MagentoRestOperator(
        task_id='set_billing_shipping_address_op',
        endpoint=f'carts/mine/shipping-information',
        method='POST',
        headers={'Authorization': f'Bearer {customer_token}'},
        data={
            "addressInformation": {
                "shipping_address": {
                    "region": "NY",
                    "region_id": 43,
                    "country_id": "US",
                    "street": ["123 Main St"],
                    "telephone": "1234567890",
                    "postcode": "12345",
                    "city": "New York",
                    "firstname": "John",
                    "lastname": "Doe",
                    "email": customer_email
                },
                "billing_address": {
                    "region": "NY",
                    "region_id": 43,
                    "country_id": "US",
                    "street": ["123 Main St"],
                    "telephone": "1234567890",
                    "postcode": "12345",
                    "city": "New York",
                    "firstname": "John",
                    "lastname": "Doe",
                    "email": customer_email
                },
                "shippingMethodCode": "flatrate",
                "shippingCarrierCode": "flatrate"
            }
        }
    )
    op.execute(context=kwargs)

@task
def set_payment_method(customer_token: str, quote_id: str, **kwargs):
    op = MagentoRestOperator(
        task_id='set_payment_method_op',
        endpoint=f'carts/mine/payment-information',
        method='POST',
        headers={'Authorization': f'Bearer {customer_token}'},
        data={
            "paymentMethod": {
                "method": "checkmo"
            }
        }
    )
    order_id =op.execute(context=kwargs)
    return order_id

@task
def create_invoice(order_id: str, **kwargs):
    op = MagentoRestOperator(
        task_id='create_invoice_op',
        endpoint=f'order/{order_id}/invoice',
        method='POST',
        data={
            "capture": True,
            "notify": True,
            "appendComment": True,
            "comment": {
                "comment": "Invoice created",
                "is_visible_on_front": 0
            }
        }
    )
    op.execute(context=kwargs)

@task
def create_shipment(order_id: str, **kwargs):
    op = MagentoRestOperator(
        task_id='create_shipment_op',
        endpoint=f'order/{order_id}/ship',
        method='POST',
        data={
            "notify": True,
            "appendComment": True,
            "comment": {
                "comment": "Shipment created",
                "is_visible_on_front": 0
            },
            "tracks": [
                {
                    "track_number": "123456",
                    "title": "Carrier",
                    "carrier_code": "custom"
                }
            ]
        }
    )
    op.execute(context=kwargs)

with dag:
    # Define the product SKUs and prices
    sku_1 = 'product_sku_1'
    sku_2 = 'product_sku_2'
    price_1 = 100.00
    price_2 = 150.00

    # Task 1: Check if customer exists
    customer_exists = check_customer_exists(customer_email)

    # Task 2: Create customer if not exists
    create_customer_task = create_customer(email=customer_email, password=customer_password)

    # Task 3: Generate customer token
    generate_customer_token_task = generate_customer_token(email=customer_email, password=customer_password)

    # Task 4: Create cart
    create_cart_task = create_cart(customer_token=generate_customer_token_task)

    # Task 5: Add products to cart
    add_product_1_to_cart = add_product_to_cart(customer_token=generate_customer_token_task, quote_id=create_cart_task, sku=sku_1, qty=1)
    add_product_2_to_cart = add_product_to_cart(customer_token=generate_customer_token_task, quote_id=create_cart_task, sku=sku_2, qty=1)

    # Task 6: Set billing and shipping address
    set_billing_shipping_address_task = set_billing_shipping_address(customer_token=generate_customer_token_task, quote_id=create_cart_task)

    # Task 7: Set payment method and place order
    set_payment_method_task = set_payment_method(customer_token=generate_customer_token_task, quote_id=create_cart_task)   

    # Task 8: Create invoice
    create_invoice_task = create_invoice(order_id=set_payment_method_task)

    # Task 9: Create shipment
    create_shipment_task = create_shipment(order_id=set_payment_method_task)

    # Define task dependencies
    customer_exists >> create_customer_task >> generate_customer_token_task
    generate_customer_token_task >> create_cart_task >> [add_product_1_to_cart, add_product_2_to_cart]
    [add_product_1_to_cart, add_product_2_to_cart] >> set_billing_shipping_address_task >> set_payment_method_task 
    set_payment_method_task >> [create_invoice_task, create_shipment_task]




from setuptools import setup, find_packages

setup(
    name='apache-airflow-provider-magento',
    version='0.4',
    description="custom provider for Magento 2 - Apache Airflow",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="Sivakumar Koduru",
    author_email="sivajik34@gmail.com",
    url="https://github.com/sivajik34/magento-airflow",    
    install_requires=[
        "apache-airflow>=2.8.0",
        "requests_oauthlib",
        "requests"
    ],
    packages=find_packages(),
    python_requires='>=3.8'  
    
)




from __future__ import annotations

from datetime import datetime

from airflow.models import DAG
from airflow.operators.python import PythonOperator
from apache_airflow_provider_magento.operators.customer import CreateCustomerOperator

def process_customer_creation(**kwargs):
    ti = kwargs["ti"]
    customer_response = ti.xcom_pull(task_ids="create_customer_task", key="magento_customer")

    if customer_response:
        # Process the response
        print("Customer created successfully:", customer_response)
    else:
        print("Customer creation failed or no response received.")

dag = DAG(
    dag_id="magento_create_customer_dag",
    start_date=datetime(2024, 8, 19),
    schedule_interval="@daily",
)

customer_data = {
  "customer": {
    "email": "jdoe34@example.com",
    "firstname": "Jane",
    "lastname": "Doe",
    "addresses": [
      {
        "defaultShipping": True,
        "defaultBilling": True,
        "firstname": "Jane",
        "lastname": "Doe",
        "region": {
          "regionCode": "NY",
          "region": "New York",
          "regionId": 43
        },
        "postcode": "10755",
        "street": [
          "123 Oak Ave"
        ],
        "city": "Purchase",
        "telephone": "512-555-1111",
        "countryId": "US"
      }
    ]
  },
  "password": "Password1"
}


create_customer_task = CreateCustomerOperator(
    task_id="create_customer_task",
    magento_conn_id="magento_default",
    customer_data=customer_data,
    dag=dag,
)

process_customer_creation_task = PythonOperator(
    task_id="process_customer_creation_task",
    python_callable=process_customer_creation,
    provide_context=True,
    dag=dag,
)

create_customer_task >> process_customer_creation_task

